#!/usr/lib/env python
# -*- encoding:utf-8 -*-

import struct
import cStringIO


PROTOCOL_VERSION = 0x01
PROTOCOL_HEADER_LENGTH = 9


class Protocol(object):
    """
        网络交互协议
    """
    def __init__(self):
        # 包长度 符号整数 4字节
        self.length = 0
        # 协议版本号 无符号整数 1字节
        self.version = PROTOCOL_VERSION
        # 包序列号 无符号整数 2字节
        self.serial = 0x01
        # 协议代码 无符号整数 2字节
        self.command = 0x00
        # 协议内容
        self.package = ""


    @property
    def length(self):
        return PROTOCOL_HEADER_LENGTH + len(self.package)


def serialize(protocol):
    """
        序列化协议
        protocol: 待序列化协议
    """
    return struct.pack("!IBHH%ds" % len(protocol.package), \
            protocol.length, protocol.version, protocol.serial, protocol.command, protocol.package)


def deserialize(p_buffer):
    """
        反序列化协议
        p_buffer: 待反序列化流
        package_class: 协议内容解包函数
    """
    # 可能存在粘包，循环解粘包
    with cStringIO.StringIO(p_buffer) as sio:
        while(sio.tell() < sio.len):
            # 读取包长度
            length = sruct.unpack("!I", sio.read(4))
            
            # 反序列化协议
            version, serial, command, package = struct.unpack(
                    "!BHH%ds" % length - PROTOCOL_HEADER_LENGTH, sio.read(length - 4))
            
            protocol = Protocol()
            protocol.length = length
            protocol.version = version
            protocol.serial = serial
            protocol.command = command
            protocol.package = package

            yield protocol
