#!/usr/lib/env python
# -*- encoding:utf-8 -*-

from twisted.internet.protocol import ServerFactory, Protocol


class ServerProtocol(Protocol):
    def connectionMade(self):
        if(self.factory.client_connect_callback):
            self.factory.client_connect_callback(self)


    def connectionLost(self, reason):
        if(self.factory.client_lost_callback):
            self.factory.client_lost_callback(self, reason)


    def dataReceived(self, data):
        if(self.factory.data_received_callback):
            self.factory.data_received_callback(self, data)
        

class Server(ServerFactory):
    """
        TCP服务器
    """
    protocol = ServerProtocol

    def __init__(self, port=8080):
        self.port = port
        self.client_connect_callback = None
        self.client_lost_callback = None
        self.data_received_callback = None


    def when_client_connect(self, callback):
        """
            注册客户端连接回调
        """
        self.client_connect_callback = callback


    def when_client_lost(self, callback):
        """
            注册客户端断开连接回调
        """
        self.client_lost_callback = callback


    def when_data_received(self, callback):
        """
            注册数据接收回调
        """
        self.data_received_callback = callback


    def start(self):
        """
            启动TCP监听
        """
        from twisted.internet import reactor
        reactor.listenTCP(self.port, self)
        reactor.run()


if __name__ == "__main__":
    def client_connect(client):
        print "connect"

    def client_lost(client, reason):
        print "lost"

    def data_received(client, data):
        print data

    s = Server()
    s.when_client_connect(client_connect)
    s.when_client_lost(client_lost)
    s.when_data_received(data_received)
    s.start()
